@@ -109,6 +109,52 @@ class TestUndercloud(BaseTestCase):
         mock_die_tuskar_die.assert_called_once()
         mock_run_validation_groups.assert_called_once()
 
+    @mock.patch('instack_undercloud.undercloud._handle_upgrade_fact')
+    @mock.patch('instack_undercloud.undercloud._configure_logging')
+    @mock.patch('instack_undercloud.undercloud._validate_configuration')
+    @mock.patch('instack_undercloud.undercloud._run_command')
+    @mock.patch('instack_undercloud.undercloud._post_config')
+    @mock.patch('instack_undercloud.undercloud._run_orc')
+    @mock.patch('instack_undercloud.undercloud._run_yum_update')
+    @mock.patch('instack_undercloud.undercloud._run_yum_clean_all')
+    @mock.patch('instack_undercloud.undercloud._run_instack')
+    @mock.patch('instack_undercloud.undercloud._generate_environment')
+    @mock.patch('instack_undercloud.undercloud._load_config')
+    @mock.patch('instack_undercloud.undercloud._die_tuskar_die')
+    @mock.patch('instack_undercloud.undercloud._run_validation_groups')
+    def test_install_upgrade_hieradata(self, mock_run_validation_groups,
+                                       mock_die_tuskar_die, mock_load_config,
+                                       mock_generate_environment,
+                                       mock_run_instack,
+                                       mock_run_yum_clean_all,
+                                       mock_run_yum_update, mock_run_orc,
+                                       mock_post_config, mock_run_command,
+                                       mock_validate_configuration,
+                                       mock_configure_logging,
+                                       mock_upgrade_fact):
+        conf = config_fixture.Config()
+        self.useFixture(conf)
+        conf.config(hieradata_override='override.yaml')
+        with open(os.path.expanduser('~/override.yaml'), 'w') as f:
+            f.write('Something\n')
+        fake_env = mock.MagicMock()
+        mock_generate_environment.return_value = fake_env
+        undercloud.install('.', upgrade=True)
+        self.assertTrue(mock_validate_configuration.called)
+        mock_generate_environment.assert_called_with('.')
+        mock_run_instack.assert_called_with(fake_env)
+        mock_run_orc.assert_called_with(fake_env)
+        mock_run_command.assert_called_with(
+            ['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
+        self.assertNotIn(
+            mock.call(
+                ['sudo', 'cp', 'override.yaml',
+                 '/etc/puppet/hieradata/override.yaml']),
+            mock_run_command.mock_calls)
+        mock_upgrade_fact.assert_called_with(True)
+        mock_die_tuskar_die.assert_called_once()
+        mock_run_validation_groups.assert_called_once()
+
     @mock.patch('instack_undercloud.undercloud._configure_logging')
     def test_install_exception(self, mock_configure_logging):
         mock_configure_logging.side_effect = RuntimeError('foo')
@@ -436,19 +482,70 @@ class TestGenerateEnvironment(BaseTestCase):
         drivers = json.loads(env['ENABLED_DRIVERS'])
         self.assertEqual(sorted(drivers), ['pxe_drac', 'pxe_ilo',
                                            'pxe_ipmitool'])
+        hw_types = json.loads(env['ENABLED_HARDWARE_TYPES'])
+        self.assertEqual(sorted(hw_types), ['idrac', 'ilo', 'ipmi', 'redfish'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_BOOT_INTERFACES'])),
+            ['ilo-pxe', 'pxe'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_POWER_INTERFACES'])),
+            ['fake', 'idrac', 'ilo', 'ipmitool', 'redfish'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_MANAGEMENT_INTERFACES'])),
+            ['fake', 'idrac', 'ilo', 'ipmitool', 'redfish'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_RAID_INTERFACES'])),
+            ['idrac', 'no-raid'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_VENDOR_INTERFACES'])),
+            ['idrac', 'ipmitool', 'no-vendor'])
         self.assertEqual(env['INSPECTION_NODE_NOT_FOUND_HOOK'], '')
 
+    def test_all_hardware_types(self):
+        conf = config_fixture.Config()
+        self.useFixture(conf)
+        conf.config(enabled_hardware_types=['ipmi', 'redfish', 'ilo',
+                                            'idrac', 'irmc', 'snmp',
+                                            'cisco-ucs-managed',
+                                            'cisco-ucs-standalone'])
+        env = undercloud._generate_environment('.')
+        # The list is generated from a set, so we can't rely on ordering.
+        # Instead make sure that it looks like a valid list by parsing it.
+        hw_types = json.loads(env['ENABLED_HARDWARE_TYPES'])
+        self.assertEqual(sorted(hw_types), ['cisco-ucs-managed',
+                                            'cisco-ucs-standalone',
+                                            'idrac', 'ilo', 'ipmi', 'irmc',
+                                            'redfish', 'snmp'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_BOOT_INTERFACES'])),
+            ['ilo-pxe', 'irmc-pxe', 'pxe'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_POWER_INTERFACES'])),
+            ['cimc', 'fake', 'idrac', 'ilo', 'ipmitool', 'irmc',
+             'redfish', 'snmp', 'ucsm'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_MANAGEMENT_INTERFACES'])),
+            ['cimc', 'fake', 'idrac', 'ilo', 'ipmitool', 'irmc',
+             'redfish', 'ucsm'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_RAID_INTERFACES'])),
+            ['idrac', 'no-raid'])
+        self.assertEqual(
+            sorted(json.loads(env['ENABLED_VENDOR_INTERFACES'])),
+            ['idrac', 'ipmitool', 'no-vendor'])
+
     def test_enabled_discovery(self):
         conf = config_fixture.Config()
         self.useFixture(conf)
         conf.config(enable_node_discovery=True,
-                    discovery_default_driver='foobar')
+                    discovery_default_driver='pxe_foobar')
         env = undercloud._generate_environment('.')
         # The list is generated from a set, so we can't rely on ordering.
         # Instead make sure that it looks like a valid list by parsing it.
         drivers = json.loads(env['ENABLED_DRIVERS'])
-        # Discovery requires enabling the default driver
-        self.assertEqual(sorted(drivers), ['foobar', 'pxe_drac', 'pxe_ilo',
+        # Discovery requires enabling the default driver. The pxe_ prefix
+        # designates a classic driver.
+        self.assertEqual(sorted(drivers), ['pxe_drac', 'pxe_foobar', 'pxe_ilo',
                                            'pxe_ipmitool'])
         self.assertEqual(env['INSPECTION_NODE_NOT_FOUND_HOOK'], 'enroll')
 
@@ -457,17 +554,15 @@ class TestGenerateEnvironment(BaseTestCase):
         self.useFixture(conf)
         conf.config(enable_node_discovery=True,
                     discovery_default_driver='foobar',
-                    enabled_hardware_types=['ipmi', 'foobar'])
+                    enabled_hardware_types=['ipmi', 'something'])
         env = undercloud._generate_environment('.')
         # The list is generated from a set, so we can't rely on ordering.
         # Instead make sure that it looks like a valid list by parsing it.
         drivers = json.loads(env['ENABLED_DRIVERS'])
         hw_types = json.loads(env['ENABLED_HARDWARE_TYPES'])
-        # The driver is already in hardware types, so we don't try adding it to
-        # the driver list.
         self.assertEqual(sorted(drivers), ['pxe_drac', 'pxe_ilo',
                                            'pxe_ipmitool'])
-        self.assertEqual(sorted(hw_types), ['foobar', 'ipmi'])
+        self.assertEqual(sorted(hw_types), ['foobar', 'ipmi', 'something'])
 
     def test_docker_registry_mirror(self):
         conf = config_fixture.Config()
@@ -482,7 +577,7 @@ class TestGenerateEnvironment(BaseTestCase):
         env = undercloud._generate_environment('.')
         endpoint_vars = {k: v for (k, v) in env.items()
                          if k.startswith('UNDERCLOUD_ENDPOINT')}
-        self.assertEqual(93, len(endpoint_vars))
+        self.assertEqual(90, len(endpoint_vars))
         # Spot check one service
         self.assertEqual('http://192.168.24.1:5000',
                          env['UNDERCLOUD_ENDPOINT_KEYSTONE_PUBLIC'])
@@ -539,10 +634,18 @@ class TestGenerateEnvironment(BaseTestCase):
     def test_relative_cert_path(self):
         conf = config_fixture.Config()
         self.useFixture(conf)
-        conf.config(undercloud_service_certificate='test.pem')
-        env = undercloud._generate_environment('.')
-        self.assertEqual(os.path.join(os.getcwd(), 'test.pem'),
-                         env['UNDERCLOUD_SERVICE_CERTIFICATE'])
+        [cert] = self.create_tempfiles([('test', 'foo')], '.pem')
+        rel_cert = os.path.basename(cert)
+        cert_path = os.path.dirname(cert)
+        cur_dir = os.getcwd()
+        try:
+            os.chdir(cert_path)
+            conf.config(undercloud_service_certificate=rel_cert)
+            env = undercloud._generate_environment('.')
+            self.assertEqual(os.path.join(os.getcwd(), rel_cert),
+                             env['UNDERCLOUD_SERVICE_CERTIFICATE'])
+        finally:
+            os.chdir(cur_dir)
 
     def test_no_cert_path(self):
         env = undercloud._generate_environment('.')
@@ -1058,10 +1161,10 @@ class TestPostConfig(base.BaseTestCase):
         mock_heat = mock.Mock()
         mock_heat.stacks.list.return_value = stacks
         undercloud._migrate_to_convergence(mock_heat)
-        self.assertEqual([mock.call(['sudo', 'heat-manage',
+        self.assertEqual([mock.call(['sudo', '-E', 'heat-manage',
                                      'migrate_convergence_1', '1'],
                                     name='heat-manage'),
-                          mock.call(['sudo', 'heat-manage',
+                          mock.call(['sudo', '-E', 'heat-manage',
                                      'migrate_convergence_1', '2'],
                                     name='heat-manage')],
                          mock_run_command.mock_calls)
@@ -1100,10 +1203,9 @@ class TestPostConfig(base.BaseTestCase):
     @mock.patch('os.listdir')
     @mock.patch('instack_undercloud.undercloud._create_mistral_config_'
                 'environment')
-    @mock.patch('instack_undercloud.undercloud._migrate_plans')
     @mock.patch('instack_undercloud.undercloud._create_default_plan')
-    def test_post_config_mistral(self, mock_create, mock_migrate, mock_cmce,
-                                 mock_listdir, mock_isfile):
+    def test_post_config_mistral(self, mock_create, mock_cmce, mock_listdir,
+                                 mock_isfile):
         instack_env = {}
         mock_mistral = mock.Mock()
         mock_swift = mock.Mock()
@@ -1141,19 +1243,15 @@ class TestPostConfig(base.BaseTestCase):
                                    '/bar.yaml')],
                          mock_mistral.workbooks.create.mock_calls)
         mock_cmce.assert_called_once_with(instack_env, mock_mistral)
-        mock_migrate.assert_called_once_with(mock_mistral, mock_swift,
-                                             ['hut8'])
         mock_create.assert_called_once_with(mock_mistral, ['hut8'])
 
     @mock.patch('os.path.isfile', return_value=True)
     @mock.patch('os.listdir')
     @mock.patch('instack_undercloud.undercloud._create_mistral_config_'
                 'environment')
-    @mock.patch('instack_undercloud.undercloud._migrate_plans')
     @mock.patch('instack_undercloud.undercloud._create_default_plan')
-    def test_post_config_mistral_with_tags(self, mock_create, mock_migrate,
-                                           mock_cmce, mock_listdir,
-                                           mock_isfile):
+    def test_post_config_mistral_with_tags(self, mock_create, mock_cmce,
+                                           mock_listdir, mock_isfile):
         instack_env = {}
         mock_mistral = mock.Mock()
         mock_swift = mock.Mock()
@@ -1190,8 +1288,6 @@ class TestPostConfig(base.BaseTestCase):
                                    '/bar.yaml')],
                          mock_mistral.workbooks.create.mock_calls)
         mock_cmce.assert_called_once_with(instack_env, mock_mistral)
-        mock_migrate.assert_called_once_with(mock_mistral, mock_swift,
-                                             ['hut8'])
         mock_create.assert_called_once_with(mock_mistral, ['hut8'])
 
 
