@@ -29,14 +29,12 @@ import sys
 import tempfile
 import time
 import uuid
-import yaml
 
 from ironicclient import client as ir_client
 from keystoneauth1 import session
 from keystoneauth1 import exceptions as ks_exceptions
 from keystoneclient import discover
 import keystoneauth1.identity.generic as ks_auth
-from mistralclient.api import base as mistralclient_base
 from mistralclient.api import client as mistralclient
 from novaclient import client as novaclient
 from novaclient import exceptions
@@ -291,10 +289,11 @@ _opts = [
                       'information for newly enrolled nodes.')
                 ),
     cfg.StrOpt('discovery_default_driver',
-               default='pxe_ipmitool',
-               help=('The default driver to use for newly discovered nodes '
-                     '(requires enable_node_discovery set to True). This '
-                     'driver is automatically added to enabled_drivers.')
+               default='ipmi',
+               help=('The default driver or hardware type to use for newly '
+                     'discovered nodes (requires enable_node_discovery set to '
+                     'True). It is automatically added to enabled_drivers '
+                     'or enabled_hardware_types accordingly.')
                ),
     cfg.BoolOpt('undercloud_debug',
                 default=True,
@@ -328,21 +327,6 @@ _opts = [
                 default=False,
                 help=('Whether to install the Volume service. It is not '
                       'currently used in the undercloud.')),
-    cfg.BoolOpt('enable_legacy_ceilometer_api',
-                default=False,
-                help=('Whether to enable legacy ceilometer api '
-                      'in the Undercloud. '
-                      'Note: Ceilometer API has been deprecated and will be '
-                      'removed in future release. Please consider moving to '
-                      'gnocchi/Aodh/Panko API instead.')
-                ),
-    cfg.BoolOpt('enable_legacy_ceilometer_collector',
-                default=False,
-                help=('Whether to enable legacy ceilometer collector '
-                      'in the Undercloud. '
-                      'Note: Ceilometer collector has been deprecated and '
-                      'will be removed in future release.')
-                ),
     cfg.BoolOpt('enable_novajoin',
                 default=False,
                 help=('Whether to install novajoin metadata service in '
@@ -377,9 +361,12 @@ _opts = [
                       'between deployments and after the introspection.')),
     cfg.ListOpt('enabled_drivers',
                 default=['pxe_ipmitool', 'pxe_drac', 'pxe_ilo'],
-                help=('List of enabled bare metal drivers.')),
+                help=('List of enabled bare metal drivers.'),
+                deprecated_for_removal=True,
+                deprecated_reason=('Please switch to hardware types and '
+                                   'the enabled_hardware_types option.')),
     cfg.ListOpt('enabled_hardware_types',
-                default=['ipmi', 'redfish'],
+                default=['ipmi', 'redfish', 'ilo', 'idrac'],
                 help=('List of enabled bare metal hardware types (next '
                       'generation drivers).')),
     cfg.StrOpt('docker_registry_mirror',
@@ -570,6 +557,15 @@ def _run_command(args, env=None, name=None):
     """
     if name is None:
         name = args[0]
+
+    if env is None:
+        env = os.environ
+    env = env.copy()
+
+    # When running a localized python script, we need to tell it that we're
+    # using utf-8 for stdout, otherwise it can't tell because of the pipe.
+    env['PYTHONIOENCODING'] = 'utf8'
+
     try:
         return subprocess.check_output(args,
                                        stderr=subprocess.STDOUT,
@@ -588,6 +584,15 @@ def _run_live_command(args, env=None, name=None):
     """
     if name is None:
         name = args[0]
+
+    if env is None:
+        env = os.environ
+    env = env.copy()
+
+    # When running a localized python script, we need to tell it that we're
+    # using utf-8 for stdout, otherwise it can't tell because of the pipe.
+    env['PYTHONIOENCODING'] = 'utf8'
+
     process = subprocess.Popen(args, env=env,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT)
@@ -855,10 +860,6 @@ def _generate_endpoints(instack_env):
             '%s://%s:%d/placement',
             {'host': public_host, 'port': 8778, 'ssl_port': 13778},
             {'host': internal_host, 'port': 8778}),
-        ('ceilometer',
-            '%s://%s:%d',
-            {'host': public_host, 'port': 8777, 'ssl_port': 13777},
-            {'host': internal_host, 'port': 8777}),
         ('keystone',
             '%s://%s:%d',
             {'host': public_host, 'port': 5000, 'ssl_port': 13000},
@@ -907,7 +908,7 @@ def _generate_endpoints(instack_env):
             {'host': internal_host, 'port': 8041}),
         ('panko',
             '%s://%s:%d',
-            {'host': public_host, 'port': 8977, 'ssl_port': 13779},
+            {'host': public_host, 'port': 8977, 'ssl_port': 13977},
             {'host': internal_host, 'port': 8977}),
         ('mistral',
             '%s://%s:%d/v2',
@@ -1059,7 +1060,8 @@ class InstackEnvironment(dict):
                     'INSPECTION_NODE_NOT_FOUND_HOOK',
                     'TRIPLEO_INSTALL_USER', 'TRIPLEO_UNDERCLOUD_CONF_FILE',
                     'TRIPLEO_UNDERCLOUD_PASSWORD_FILE',
-                    'ENABLED_POWER_INTERFACES',
+                    'ENABLED_BOOT_INTERFACES', 'ENABLED_POWER_INTERFACES',
+                    'ENABLED_RAID_INTERFACES', 'ENABLED_VENDOR_INTERFACES',
                     'ENABLED_MANAGEMENT_INTERFACES', 'SYSCTL_SETTINGS',
                     'LOCAL_IP_WRAPPED'}
     """The variables we calculate in _generate_environment call."""
@@ -1094,6 +1096,76 @@ def _generate_sysctl_settings():
     return json.dumps(sysctl_settings)
 
 
+def _is_classic_driver(name):
+    """Poor man's way to detect if something is a driver or a hardware type.
+
+    To be removed when we remove support for classic drivers.
+    """
+    return (name == 'fake' or
+            name.startswith('fake_') or
+            name.startswith('pxe_') or
+            name.startswith('agent_') or
+            name.startswith('iscsi_'))
+
+
+def _process_drivers_and_hardware_types(instack_env):
+    """Populate the environment with ironic driver information."""
+    # Ensure correct rendering of the list and uniqueness of the items
+    enabled_drivers = set(CONF.enabled_drivers)
+    enabled_hardware_types = set(CONF.enabled_hardware_types)
+    if CONF.enable_node_discovery:
+        if _is_classic_driver(CONF.discovery_default_driver):
+            if CONF.discovery_default_driver not in enabled_drivers:
+                enabled_drivers.add(CONF.discovery_default_driver)
+        else:
+            if CONF.discovery_default_driver not in enabled_hardware_types:
+                enabled_hardware_types.add(CONF.discovery_default_driver)
+        instack_env['INSPECTION_NODE_NOT_FOUND_HOOK'] = 'enroll'
+    else:
+        instack_env['INSPECTION_NODE_NOT_FOUND_HOOK'] = ''
+
+    # In most cases power and management interfaces are called the same, so we
+    # use one variable for them.
+    mgmt_interfaces = {'fake', 'ipmitool'}
+    # TODO(dtantsur): can we somehow avoid hardcoding hardware types here?
+    for hw_type in ('redfish', 'idrac', 'ilo', 'irmc'):
+        if hw_type in enabled_hardware_types:
+            mgmt_interfaces.add(hw_type)
+    for (hw_type, iface) in [('cisco-ucs-managed', 'ucsm'),
+                             ('cisco-ucs-standalone', 'cimc')]:
+        if hw_type in enabled_hardware_types:
+            mgmt_interfaces.add(iface)
+
+    # Two hardware types use non-default boot interfaces.
+    boot_interfaces = {'pxe'}
+    for hw_type in ('ilo', 'irmc'):
+        if hw_type in enabled_hardware_types:
+            boot_interfaces.add('%s-pxe' % hw_type)
+
+    raid_interfaces = {'no-raid'}
+    if 'idrac' in enabled_hardware_types:
+        raid_interfaces.add('idrac')
+
+    vendor_interfaces = {'no-vendor'}
+    for (hw_type, iface) in [('ipmi', 'ipmitool'),
+                             ('idrac', 'idrac')]:
+        if hw_type in enabled_hardware_types:
+            vendor_interfaces.add(iface)
+
+    instack_env['ENABLED_DRIVERS'] = _make_list(enabled_drivers)
+    instack_env['ENABLED_HARDWARE_TYPES'] = _make_list(enabled_hardware_types)
+
+    instack_env['ENABLED_BOOT_INTERFACES'] = _make_list(boot_interfaces)
+    instack_env['ENABLED_MANAGEMENT_INTERFACES'] = _make_list(mgmt_interfaces)
+    instack_env['ENABLED_RAID_INTERFACES'] = _make_list(raid_interfaces)
+    instack_env['ENABLED_VENDOR_INTERFACES'] = _make_list(vendor_interfaces)
+
+    # The snmp hardware type uses fake management and snmp power
+    if 'snmp' in enabled_hardware_types:
+        mgmt_interfaces.add('snmp')
+    instack_env['ENABLED_POWER_INTERFACES'] = _make_list(mgmt_interfaces)
+
+
 def _generate_environment(instack_root):
     """Generate an environment dict for instack
 
@@ -1175,31 +1247,7 @@ def _generate_environment(instack_root):
 
     instack_env['INSPECTION_KERNEL_ARGS'] = ' '.join(inspection_kernel_args)
 
-    # Ensure correct rendering of the list and uniqueness of the items
-    enabled_drivers = set(CONF.enabled_drivers)
-    enabled_hardware_types = set(CONF.enabled_hardware_types)
-    if CONF.enable_node_discovery:
-        if (CONF.discovery_default_driver not in (enabled_drivers |
-                                                  enabled_hardware_types)):
-            enabled_drivers.add(CONF.discovery_default_driver)
-        instack_env['INSPECTION_NODE_NOT_FOUND_HOOK'] = 'enroll'
-    else:
-        instack_env['INSPECTION_NODE_NOT_FOUND_HOOK'] = ''
-
-    # In most cases power and management interfaces are called the same, so we
-    # use one variable for them.
-    enabled_interfaces = set()
-    if 'ipmi' in enabled_hardware_types:
-        enabled_interfaces.add('ipmitool')
-    if 'redfish' in enabled_hardware_types:
-        enabled_interfaces.add('redfish')
-
-    instack_env['ENABLED_DRIVERS'] = _make_list(enabled_drivers)
-    instack_env['ENABLED_HARDWARE_TYPES'] = _make_list(enabled_hardware_types)
-
-    enabled_interfaces = _make_list(enabled_interfaces)
-    instack_env['ENABLED_POWER_INTERFACES'] = enabled_interfaces
-    instack_env['ENABLED_MANAGEMENT_INTERFACES'] = enabled_interfaces
+    _process_drivers_and_hardware_types(instack_env)
 
     instack_env['SYSCTL_SETTINGS'] = _generate_sysctl_settings()
 
@@ -1209,9 +1257,6 @@ def _generate_environment(instack_root):
     instack_env['PUBLIC_INTERFACE_IP'] = instack_env['LOCAL_IP']
     instack_env['LOCAL_IP'] = instack_env['LOCAL_IP'].split('/')[0]
     instack_env['LOCAL_IP_WRAPPED'] = _wrap_ipv6(instack_env['LOCAL_IP'])
-    if instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']:
-        instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = os.path.abspath(
-            instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'])
     # We're not in a chroot so this doesn't make sense, and it causes weird
     # errors if it's set.
     if instack_env.get('DIB_YUM_REPO_CONF'):
@@ -1234,6 +1279,19 @@ def _generate_environment(instack_root):
         public_host = CONF.undercloud_public_host
         instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = (
             '/etc/pki/tls/certs/undercloud-%s.pem' % public_host)
+    elif instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']:
+        raw_value = instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']
+        abs_cert = os.path.abspath(raw_value)
+        if abs_cert != raw_value:
+            home_dir = os.path.expanduser('~')
+            if os.getcwd() != home_dir and os.path.exists(abs_cert):
+                LOG.warning('Using undercloud_service_certificate from '
+                            'current directory, please use an absolute path '
+                            'to remove ambiguity')
+                instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = abs_cert
+            else:
+                instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = os.path.join(
+                    home_dir, raw_value)
 
     return instack_env
 
@@ -1257,12 +1315,20 @@ def _generate_init_data(instack_env):
     context = instack_env.copy()
 
     if CONF.hieradata_override:
-        hiera_entry = os.path.splitext(
-            os.path.basename(CONF.hieradata_override))[0]
+        data_file = CONF.hieradata_override
+        hiera_entry = os.path.splitext(os.path.basename(data_file))[0]
         dst = os.path.join('/etc/puppet/hieradata',
-                           os.path.basename(CONF.hieradata_override))
+                           os.path.basename(data_file))
+        if os.path.abspath(CONF.hieradata_override) != data_file:
+            # If we don't have an absolute path, compute it
+            data_file = os.path.join(os.path.expanduser('~'), data_file)
+
+        if not os.path.exists(data_file):
+            raise RuntimeError(
+                "Could not find hieradata_override file '%s'" % data_file)
+
         _run_command(['sudo', 'mkdir', '-p', '/etc/puppet/hieradata'])
-        _run_command(['sudo', 'cp', CONF.hieradata_override, dst])
+        _run_command(['sudo', 'cp', data_file, dst])
         _run_command(['sudo', 'chmod', '0644', dst])
     else:
         hiera_entry = ''
@@ -1375,6 +1441,32 @@ def _configure_ssh_keys(nova):
             nova.keypairs.create('default', pubkey.read().rstrip())
 
 
+def _ensure_ssh_selinux_permission():
+    ssh_path = os.path.expanduser('~/.ssh')
+    try:
+        enforcing = _run_command(['getenforce'])
+        if os.path.isdir(ssh_path):
+            if 'Enforcing' in enforcing:
+                file_perms = _run_command(
+                    ['find', ssh_path, '-exec', 'ls', '-lZ', '{}', ';'])
+                wrong_perm = False
+                for line in file_perms.splitlines():
+                    if 'ssh_home_t' not in line:
+                        wrong_perm = True
+                        break
+                if wrong_perm:
+                    cmd = ['semanage',
+                           'fcontext', '-a', '-t', 'ssh_home_t',
+                           "{}(/.*)?".format(ssh_path)]
+                    _run_command(cmd)
+                    _run_command(['restorecon', '-R', ssh_path])
+    except OSError as e:
+        if e.errno == os.errno.ENOENT:
+            LOG.debug("Not a SeLinux platform")
+        else:
+            raise
+
+
 def _delete_default_flavors(nova):
     """Delete the default flavors from Nova
 
@@ -1429,7 +1521,7 @@ def _ensure_flavor(nova, existing, name, profile=None):
 
 
 def _ensure_node_resource_classes(ironic):
-    for node in ironic.node.list(limit=-1, fields=['uuid', 'resource_class']):
+    for node in ironic.node.list(limit=0, fields=['uuid', 'resource_class']):
         if node.resource_class:
             if node.resource_class != DEFAULT_NODE_RESOURCE_CLASS:
                 LOG.warning('Node %s is using a resource class %s instead '
@@ -1484,32 +1576,6 @@ def _create_mistral_config_environment(instack_env, mistral):
             }))
 
 
-def _migrate_plans(mistral, swift, plans):
-    """Migrate plan environments from Mistral to Swift."""
-    plan_env_filename = 'plan-environment.yaml'
-
-    for plan in plans:
-        headers, objects = swift.get_container(plan)
-
-        if headers.get('x-container-meta-usage-tripleo') != 'plan':
-            continue
-
-        try:
-            swift.get_object(plan, plan_env_filename)
-        except swiftclient.ClientException:
-            LOG.info('Migrating environment for plan %s to Swift.' % plan)
-            try:
-                env = mistral.environments.get(plan).variables
-            except (mistralclient_base.APIException,
-                    ks_exceptions.http.NotFound):
-                LOG.warning('Could not find plan "%s" environment in Mistral '
-                            '- nothing to migrate.' % plan)
-            else:
-                yaml_string = yaml.safe_dump(env, default_flow_style=False)
-                swift.put_object(plan, plan_env_filename, yaml_string)
-                mistral.environments.delete(plan)
-
-
 def _wait_for_mistral_execution(timeout_at, mistral, execution, message='',
                                 fail_on_error=False):
     while time.time() < timeout_at:
@@ -1657,7 +1723,6 @@ def _post_config_mistral(instack_env, mistral, swift):
     plans = [container["name"] for container in swift.get_account()[1]]
 
     _create_mistral_config_environment(instack_env, mistral)
-    _migrate_plans(mistral, swift, plans)
     _create_default_plan(mistral, plans)
     _create_logging_cron(mistral)
 
@@ -1675,7 +1740,7 @@ def _migrate_to_convergence(heat):
     """
     for stack in heat.stacks.list():
         LOG.info('Migrating stack "%s" to convergence engine', stack.id)
-        args = ['sudo', 'heat-manage', 'migrate_convergence_1', stack.id]
+        args = ['sudo', '-E', 'heat-manage', 'migrate_convergence_1', stack.id]
         _run_command(args, name='heat-manage')
         LOG.info('Finished migrating stack "%s"', stack.id)
 
@@ -1690,6 +1755,7 @@ def _post_config(instack_env, upgrade):
                                   os_ironic_api_version='1.21')
 
     _configure_ssh_keys(nova)
+    _ensure_ssh_selinux_permission()
     _delete_default_flavors(nova)
 
     _ensure_node_resource_classes(ironic)
@@ -1790,7 +1856,7 @@ def install(instack_root, upgrade=False):
             # `nova-manage db online_data_migrations` command before. This
             # could cause the post-upgrade db sync to fail. Better be safe
             # than sorry and run it before package upgrade.
-            _run_command(['sudo', '/usr/bin/nova-manage', 'db',
+            _run_command(['sudo', '-E', '/usr/bin/nova-manage', 'db',
                           'online_data_migrations'])
             # Even if we backport https://review.openstack.org/#/c/457478/
             # into stable branches of puppet-ironic, we still need a way
@@ -1836,6 +1902,11 @@ def pre_upgrade():
     _run_live_command(args, name='systemctl stop')
     LOG.info('Services stopped successfully')
 
+    args = ['sudo', 'yum', 'install', '-y', 'ansible-pacemaker']
+    LOG.info('Installing Ansible Pacemaker module')
+    _run_live_command(args, name='install ansible')
+    LOG.info('Ansible pacemaker install completed successfully')
+
     args = ['sudo', 'yum', 'update', '-y']
     LOG.info('Updating full system')
     _run_live_command(args, name='yum update')
